// Perspect.c
// Demonstrates Perspective Projection

#include <afxwin.h>

#include <windows.h>            
#include <gl\gl.h>              
#include <gl\glu.h>

static	HGLRC hRC;
static	HDC hDC;

const	int	FULLSCREEN = 1;		// 1 - full screen, 0 - window

BOOL	keys[256];

GLfloat	Qrot, Trot;

LRESULT CALLBACK WndProc(   HWND    hWnd,
							UINT    message,
							WPARAM  wParam,
							LPARAM  lParam);

void SetDCPixelFormat(HDC hDC);


GLvoid Resize(GLsizei Width, GLsizei Height)
{
	if(Height==0)
		Height = 1;
	glViewport(0, 0, Width, Height);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluPerspective(45.0f, (GLfloat)Width / (GLfloat)Height, 0.1f, 100.0f);

	glMatrixMode(GL_MODELVIEW);
}

GLvoid Initial(GLsizei Width, GLsizei Height)
{
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

	glClearDepth(1.0);
	glDepthFunc(GL_LESS);
	glEnable(GL_DEPTH_TEST);

	glShadeModel(GL_SMOOTH);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluPerspective(45.0f, (GLfloat)Width / (GLfloat)Height, 0.1f, 100.0f);
	glMatrixMode(GL_MODELVIEW);
}

void Draw(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	glTranslatef(-1.5f, 0.0f, -5.0f);
	glRotatef(Trot, 0.0f, 1.0f, 1.0f);

	glBegin(GL_TRIANGLES);
		glColor3f(1.0f,0.0f,0.0f);			// Red
		glVertex3f( 0.0f, 1.0f, 0.0f);			// Top Of Triangle (Front)
		glColor3f(0.0f,1.0f,0.0f);			// Green
		glVertex3f(-1.0f,-1.0f, 1.0f);			// Left Of Triangle (Front)
		glColor3f(0.0f,0.0f,1.0f);			// Blue
		glVertex3f( 1.0f,-1.0f, 1.0f);			// Right Of Triangle (Front)

		glColor3f(1.0f,0.0f,0.0f);		// Red
		glVertex3f( 0.0f, 1.0f, 0.0f);	// Top Of Triangle (Right)
		glColor3f(0.0f,0.0f,1.0f);		// Blue
		glVertex3f( 1.0f,-1.0f, 1.0f);	// Left Of Triangle (Right)
		glColor3f(0.0f,1.0f,0.0f);		// Green
		glVertex3f( 1.0f,-1.0f, -1.0f);	// Right Of Triangle (Right)

		glColor3f(1.0f,0.0f,0.0f);		// Red
		glVertex3f( 0.0f, 1.0f, 0.0f);	// Top Of Triangle (Back)
		glColor3f(0.0f,1.0f,0.0f);		// Green
		glVertex3f( 1.0f,-1.0f, -1.0f);	// Left Of Triangle (Back)
		glColor3f(0.0f,0.0f,1.0f);		// Blue
		glVertex3f(-1.0f,-1.0f, -1.0f);	// Right Of Triangle (Back)

		glColor3f(1.0f,0.0f,0.0f);		// Red
		glVertex3f( 0.0f, 1.0f, 0.0f);	// Top Of Triangle (Left)
		glColor3f(0.0f,0.0f,1.0f);		// Blue
		glVertex3f(-1.0f,-1.0f,-1.0f);	// Left Of Triangle (Left)
		glColor3f(0.0f,1.0f,0.0f);		// Green
		glVertex3f(-1.0f,-1.0f, 1.0f);	// Right Of Triangle (Left)
	glEnd();

	glLoadIdentity();
	glTranslatef(1.5f, 0.0f, -5.0f);
	glRotatef(Qrot, 1.0f, 1.0f, 0.0f);

	glBegin(GL_QUADS);
		glColor3f(0.0f,1.0f,0.0f);			// Set The Color To Blue
		glVertex3f( 1.0f, 1.0f,-1.0f);			// Top Right Of The Quad (Top)
		glVertex3f(-1.0f, 1.0f,-1.0f);			// Top Left Of The Quad (Top)
		glVertex3f(-1.0f, 1.0f, 1.0f);			// Bottom Left Of The Quad (Top)
		glVertex3f( 1.0f, 1.0f, 1.0f);			// Bottom Right Of The Quad (Top)

		glColor3f(1.0f,0.5f,0.0f);			// Set The Color To Orange
		glVertex3f( 1.0f,-1.0f, 1.0f);			// Top Right Of The Quad (Bottom)
		glVertex3f(-1.0f,-1.0f, 1.0f);			// Top Left Of The Quad (Bottom)
		glVertex3f(-1.0f,-1.0f,-1.0f);			// Bottom Left Of The Quad (Bottom)
		glVertex3f( 1.0f,-1.0f,-1.0f);			// Bottom Right Of The Quad (Bottom)

		glColor3f(1.0f,0.0f,0.0f);			// Set The Color To Red
		glVertex3f( 1.0f, 1.0f, 1.0f);			// Top Right Of The Quad (Front)
		glVertex3f(-1.0f, 1.0f, 1.0f);			// Top Left Of The Quad (Front)
		glVertex3f(-1.0f,-1.0f, 1.0f);			// Bottom Left Of The Quad (Front)
		glVertex3f( 1.0f,-1.0f, 1.0f);			// Bottom Right Of The Quad (Front)

		glColor3f(1.0f,1.0f,0.0f);			// Set The Color To Yellow
		glVertex3f( 1.0f,-1.0f,-1.0f);			// Top Right Of The Quad (Back)
		glVertex3f(-1.0f,-1.0f,-1.0f);			// Top Left Of The Quad (Back)
		glVertex3f(-1.0f, 1.0f,-1.0f);			// Bottom Left Of The Quad (Back)
		glVertex3f( 1.0f, 1.0f,-1.0f);			// Bottom Right Of The Quad (Back)

		glColor3f(0.0f,0.0f,1.0f);			// Set The Color To Blue
		glVertex3f(-1.0f, 1.0f, 1.0f);			// Top Right Of The Quad (Left)
		glVertex3f(-1.0f, 1.0f,-1.0f);			// Top Left Of The Quad (Left)
		glVertex3f(-1.0f,-1.0f,-1.0f);			// Bottom Left Of The Quad (Left)
		glVertex3f(-1.0f,-1.0f, 1.0f);			// Bottom Right Of The Quad (Left)

		glColor3f(1.0f,0.0f,1.0f);			// Set The Color To Violet
		glVertex3f( 1.0f, 1.0f,-1.0f);			// Top Right Of The Quad (Right)
		glVertex3f( 1.0f, 1.0f, 1.0f);			// Top Left Of The Quad (Right)
		glVertex3f( 1.0f,-1.0f, 1.0f);			// Bottom Left Of The Quad (Right)
		glVertex3f( 1.0f,-1.0f,-1.0f);			// Bottom Right Of The Quad (Right)
	glEnd();

	Trot += 0.3f;
	Qrot -= 0.3f;
}

int APIENTRY WinMain(   HINSTANCE       hInst, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	MSG                     msg;   
	WNDCLASS        wc;            
	HWND            hWnd;          

	wc.style                = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wc.lpfnWndProc          = (WNDPROC) WndProc;
	wc.cbClsExtra           = 0;
	wc.cbWndExtra           = 0;
	wc.hInstance            = hInst;
	wc.hIcon                = NULL;
	wc.hCursor              = LoadCursor(NULL, IDC_ARROW);
	
	wc.hbrBackground        = NULL;         
	
	wc.lpszMenuName         = NULL;
	wc.lpszClassName        = "Skeleton";

	if(RegisterClass(&wc) == 0)
		return FALSE;

	if(FULLSCREEN==0) 
	{
		hWnd = CreateWindow(
					"Skeleton",
					"Skeleton",
					
					WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
					50, 50,
					700, 400,
					NULL,
					NULL,
					hInst,
					NULL);
	} else {
		hWnd = CreateWindow(
					"Skeleton",
					"Skeleton",
					
					WS_POPUP | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
					0, 0,
					800, 600,
					NULL,
					NULL,
					hInst,
					NULL);
	}

	if(hWnd == NULL)
		return FALSE;

	if(FULLSCREEN==1)
	{	
		DEVMODE dmScreenSettings;

		memset(&dmScreenSettings, 0, sizeof(DEVMODE));
		dmScreenSettings.dmSize			= sizeof(DEVMODE);
		dmScreenSettings.dmPelsWidth	= 800;
		dmScreenSettings.dmPelsHeight	= 600;
		dmScreenSettings.dmFields		= DM_PELSWIDTH | DM_PELSHEIGHT;
		ChangeDisplaySettings(&dmScreenSettings, CDS_FULLSCREEN);
	}

	ShowWindow(hWnd,SW_SHOW);
	UpdateWindow(hWnd);
	SetFocus(hWnd);

	while(1)
	{
		while( PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
		{
			if( GetMessage(&msg, NULL, 0, 0) ) 
			{
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
			else
			{
				return TRUE;
			}
		}
		Draw();
		SwapBuffers(hDC);
	}	
}

LRESULT CALLBACK WndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	RECT Screen;
	GLuint PixelFormat;

	static PIXELFORMATDESCRIPTOR pfd =
	{
		sizeof(PIXELFORMATDESCRIPTOR),		// 
		1,									//  ?
		PFD_DRAW_TO_WINDOW|					// format must support Window
		PFD_SUPPORT_OPENGL|					// format must support OpenGL
		PFD_DOUBLEBUFFER,					// must support double buffer
		PFD_TYPE_RGBA,						//  RGBA 
		16,									// 16Bit color depth
		0, 0, 0, 0, 0, 0,					// Color bits ignored ?
		0,									// No Alpha buffer
		0,									// shift bit ignored
		0,									// no accumulation buffer
		0, 0, 0, 0,							// accumulation buffer bits ignored
		16,									// 16bit z-buffer (depth buffer)
		0,									// no stencil buffer
		0,									// no auxiliary buffer
		PFD_MAIN_PLANE,						// main drawing layer
		0,									// reserved
		0, 0, 0								// layer mask ignored
	};

	switch(msg)
	{
	case WM_CREATE:
		hDC = GetDC(hWnd);
		PixelFormat = ChoosePixelFormat(hDC, &pfd);

		if(!PixelFormat){
			MessageBox(0, "Can't find suitable PixelFormat","Error", MB_OK|MB_ICONERROR);
			PostQuitMessage(0);
			break;
		}

		if(!SetPixelFormat(hDC, PixelFormat, &pfd)) {
			MessageBox(0, "Can't set The PixelFormat","Error",MB_OK|MB_ICONERROR);
			PostQuitMessage(0);
			break;
		}
		hRC = wglCreateContext(hDC);
		if(!hRC) {
			MessageBox(0, "Can't Create Render Device Context","Error",MB_OK|MB_ICONERROR);
			PostQuitMessage(0);
			break;
		}
		if(!wglMakeCurrent(hDC, hRC)) {
			MessageBox(0, "Can't set current Render Device Context","Error",MB_OK|MB_ICONERROR);
			PostQuitMessage(0);
			break;
		}

		GetClientRect(hWnd, &Screen);
		Initial(Screen.right, Screen.bottom);
		break;
	case WM_CLOSE:
	case WM_DESTROY:
		ChangeDisplaySettings(NULL, 0);

		wglMakeCurrent(hDC, NULL);
		wglDeleteContext(hRC);
		ReleaseDC(hWnd, hDC);

		PostQuitMessage(0);
		break;
	case WM_KEYDOWN:
		if(wParam == VK_ESCAPE)
			SendMessage(hWnd, WM_DESTROY, wParam, lParam);
		break;
	case WM_SIZE:
		Resize(LOWORD(lParam), HIWORD(lParam));
		break;
	default:
		return( DefWindowProc(hWnd, msg, wParam, lParam));
	}
	return(0);
}
